@@ -5,15 +5,17 @@
 """
 Useful utilities for working with bitarrays.
 """
+from __future__ import absolute_import
+
 import sys
 import heapq
 import binascii
 
-from bitarray import bitarray, bits2bytes, _bitarray, get_default_endian
+from bitarray import bitarray, bits2bytes, get_default_endian
 
 from bitarray._util import (count_n, rindex,
                             count_and, count_or, count_xor, subset,
-                            _set_babt)
+                            _swap_hilo_bytes, _set_bato)
 
 
 __all__ = ['zeros', 'make_endian', 'rindex', 'strip', 'count_n',
@@ -21,9 +23,9 @@ __all__ = ['zeros', 'make_endian', 'rindex', 'strip', 'count_n',
            'ba2hex', 'hex2ba', 'ba2int', 'int2ba', 'huffman_code']
 
 
-# tell the _util extension what the bitarray base type is, such that it can
-# check for instances thereof when checking for bitarray type
-_set_babt(_bitarray)
+# tell the _util extension what the bitarray type object is, such that it
+# can check for instances thereof
+_set_bato(bitarray)
 
 _is_py2 = bool(sys.version_info[0] == 2)
 
@@ -53,14 +55,14 @@ one.
 Otherwise (endianness is already `endian`) the original bitarray is returned
 unchanged.
 """
-    if not isinstance(a, _bitarray):
+    if not isinstance(a, bitarray):
         raise TypeError("bitarray expected")
 
     if a.endian() == endian:
         return a
 
     b = bitarray(a, endian)
-    la = a.length()
+    la = len(a)
     if la == 0:
         return b
 
@@ -77,7 +79,7 @@ def strip(a, mode='right'):
 Strip zeros from left, right or both ends.
 Allowed values for mode are the strings: `left`, `right`, `both`
 """
-    if not isinstance(a, _bitarray):
+    if not isinstance(a, bitarray):
         raise TypeError("bitarray expected")
     if not isinstance(mode, str):
         raise TypeError("string expected for mode")
@@ -91,7 +93,7 @@ Allowed values for mode are the strings: `left`, `right`, `both`
         except ValueError:
             return bitarray(endian=a.endian())
 
-    last = a.length() - 1
+    last = len(a) - 1
     if mode in ('right', 'both'):
         try:
             last = rindex(a)
@@ -101,24 +103,16 @@ Allowed values for mode are the strings: `left`, `right`, `both`
     return a[first:last + 1]
 
 
-# translate table which swaps the 4 highest with the 4 lowest bits"
-_swap_hilo_list = [16 * (i % 16) + (i // 16) for i in range(256)]
-if _is_py2:
-    _swap_hilo_bytes = b''.join(chr(i) for i in _swap_hilo_list)
-else:
-    _swap_hilo_bytes = bytes(_swap_hilo_list)
-
-
 def ba2hex(a):
     """ba2hex(bitarray, /) -> hexstr
 
 Return a string containing with hexadecimal representation of
 the bitarray (which has to be multiple of 4 in length).
 """
-    if not isinstance(a, _bitarray):
+    if not isinstance(a, bitarray):
         raise TypeError("bitarray expected")
 
-    la = a.length()
+    la = len(a)
     if la % 4:
         raise ValueError("bitarray length not multiple of 4")
     if la % 8:
@@ -159,26 +153,26 @@ hexstr may contain any number of hex digits (upper or lower case).
     return a
 
 
-def ba2int(a):
-    """ba2int(bitarray, /) -> int
+def ba2int(a, signed=False):
+    """ba2int(bitarray, /, signed=False) -> int
 
 Convert the given bitarray into an integer.
 The bit-endianness of the bitarray is respected.
+`signed` indicates whether two's complement is used to represent the integer.
 """
-    if not isinstance(a, _bitarray):
+    if not isinstance(a, bitarray):
         raise TypeError("bitarray expected")
     if not a:
         raise ValueError("non-empty bitarray expected")
 
-    endian = a.endian()
-    big_endian = bool(endian == 'big')
-    if a.length() % 8:
-        # pad with leading zeros, such that length is multiple of 8
+    big_endian = bool(a.endian() == 'big')
+    la = len(a)
+    if la % 8:  # pad with leading / trailing zeros
         if big_endian:
-            a = zeros(8 - a.length() % 8, 'big') + a
+            a = zeros(8 - la % 8, 'big') + a
         else:
-            a = a + zeros(8 - a.length() % 8, 'little')
-    assert a.length() % 8 == 0
+            a = a + zeros(8 - la % 8, 'little')
+    assert len(a) % 8 == 0
     b = a.tobytes()
 
     if _is_py2:
@@ -188,41 +182,50 @@ The bit-endianness of the bitarray is respected.
         for x in c:
             res |= x << 8 * j
             j += -1 if big_endian else 1
-        return res
     else: # py3
-        return int.from_bytes(b, byteorder=endian)
+        res = int.from_bytes(b, byteorder=a.endian())
+
+    if signed and res >= 1 << (la - 1):
+        res -= 1 << la
+    return res
 
 
-def int2ba(i, length=None, endian=None):
-    """int2ba(int, /, length=None, endian=None) -> bitarray
+def int2ba(i, length=None, endian=None, signed=False):
+    """int2ba(int, /, length=None, endian=None, signed=False) -> bitarray
 
-Convert the given integer into a bitarray (with given endianness,
-and no leading (big-endian) / trailing (little-endian) zeros).
-If length is provided, the result will be of this length, and an
-`OverflowError` will be raised, if the integer cannot be represented
-within length bits.
+Convert the given integer to a bitarray (with given endianness,
+and no leading (big-endian) / trailing (little-endian) zeros), unless
+the `length` of the bitarray is provided.  An `OverflowError` is raised
+if the integer is not representable with the given number of bits.
+`signed` determines whether two's complement is used to represent the integer,
+and requires `length` to be provided.
+If signed is False and a negative integer is given, an OverflowError
+is raised.
 """
     if not isinstance(i, (int, long) if _is_py2 else int):
         raise TypeError("integer expected")
-    if i < 0:
-        raise ValueError("non-negative integer expected")
     if length is not None:
         if not isinstance(length, int):
             raise TypeError("integer expected for length")
         if length <= 0:
             raise ValueError("integer larger than 0 expected for length")
-    if endian is None:
-        endian = get_default_endian()
-    if not isinstance(endian, str):
-        raise TypeError("string expected for endian")
-    if endian not in ('big', 'little'):
-        raise ValueError("endian can only be 'big' or 'little'")
+    if signed and length is None:
+        raise TypeError("signed requires length")
 
     if i == 0:
-        # there a special cases for 0 which we'd rather not deal with below
+        # there are special cases for 0 which we'd rather not deal with below
         return zeros(length or 1, endian)
 
-    big_endian = bool(endian == 'big')
+    if signed:
+        if i >= 1 << (length - 1) or i < -(1 << (length - 1)):
+            raise OverflowError("signed integer out of range")
+        if i < 0:
+            i += 1 << length
+    elif i < 0 or (length and i >= 1 << length):
+        raise OverflowError("unsigned integer out of range")
+
+    a = bitarray(0, endian or get_default_endian())
+    big_endian = bool(a.endian() == 'big')
     if _is_py2:
         c = bytearray()
         while i:
@@ -232,31 +235,19 @@ within length bits.
             c.reverse()
         b = bytes(c)
     else: # py3
-        b = i.to_bytes(bits2bytes(i.bit_length()), byteorder=endian)
+        b = i.to_bytes(bits2bytes(i.bit_length()), byteorder=a.endian())
 
-    a = bitarray(0, endian)
     a.frombytes(b)
-    la = a.length()
-    if la == length:
-        return a
-
     if length is None:
         return strip(a, 'left' if big_endian else 'right')
 
+    la = len(a)
     if la > length:
-        size = (la - a.index(1)) if big_endian else (rindex(a) + 1)
-        if size > length:
-            raise OverflowError("cannot represent %d bit integer in "
-                                "%d bits" % (size, length))
-        a = a[la - length:] if big_endian else a[:length - la]
-
+        a = a[-length:] if big_endian else a[:length]
     if la < length:
-        if big_endian:
-            a = zeros(length - la, 'big') + a
-        else:
-            a += zeros(length - la, 'little')
-
-    assert a.length() == length
+        pad = zeros(length - la, endian)
+        a = pad + a if big_endian else a + pad
+    assert len(a) == length
     return a
 
 
